###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

# Release information about openerp-web

version = "5.0.0"
release = "3.1"

description = "Web Client of OpenERP, the Enterprise Management Software"
long_description = "OpenERP Web is the web client of the OpenERP, a free enterprise management software: accounting, stock, manufacturing, project mgt..."
author = "Tiny ERP Pvt. Ltd."
email = "info@openerp.com"
copyright = "Copyright (C) 2007-TODAY Tiny ERP Pvt. Ltd."

# if it's open source, you might want to specify these
url = "http://openerp.com/"
download_url = "http://www.openerp.com/index.php?option=com_content&view=article&id=18&Itemid=28"
license = "OEPL"

# vim: ts=4 sts=4 sw=4 si et

