/**
 * Info Dialog
 */

div.infoLayer {
    display: none;

    position: absolute;
    top: 0;
    left: 0;
    z-index: 199;

    width: 100%;
    height: 100%;

    background-color: gray;
    opacity: 0.3;
}

div.infoBox {
    display: none;
    position: absolute;
    z-index: 200;

    background-color: white;
    border: 2px solid #696969;
}

div.infoBox .infoTitle {
    padding: 2px;
    background-color: #696969;
    color: white;
    font-weight: bold;
}

div.infoBox .infoDesc {
    padding: 2px;
}

div.infoBox .infoButtons {
    position: absolute;
    bottom: 0pt;
    padding: 2px;
    border-top: 2px solid #696969;
}

div.infoBox .infoButtons button {
    margin-right: 2px;
}

