#modalbox_overlay {
    display: none;

    position: absolute;
    top: 0;
    left: 0;
    z-index: 199;

    width: 100%;
    height: 100%;

    background-color: gray;
    opacity: 0.3;
}

#modalbox {
	display: none;
    position: absolute;
    z-index: 200;
    
    margin: auto;
    width: 350px;
    height: 100px;

    background-color: white;
    border: 2px solid #696969;
}

#modalbox .modalbox-title {
    padding: 2px;
    background-color: #696969;
    color: white;
    font-weight: bold;
}

#modalbox .modalbox-buttons {
    position: absolute;
    bottom: 0pt;
    padding: 2px;
    border-top: 2px solid #696969;
}

#modalbox .modalbox-buttons button {
    margin-left: 2px;
}

#modalbox .modalbox-content {
    overflow: auto;
}

/**
 * vim: sts=4 st=4 et
 *
 */
