////////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
//
// $Id$
//
// Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
//
// The OpenERP web client is distributed under the "OpenERP Public License".
// It's based on Mozilla Public License Version (MPL) 1.1 with following 
// restrictions:
//
// -   All names, links and logos of Tiny, Open ERP and Axelor must be 
//     kept as in original distribution without any changes in all software 
//     screens, especially in start-up page and the software header, even if 
//     the application source code has been changed or updated or code has been 
//     added.
//
// -   All distributions of the software must keep source code with OEPL.
// 
// -   All integrations to any other software must keep source code with OEPL.
//
// If you need commercial licence to remove this kind of restriction please
// contact us.
//
// You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
//
////////////////////////////////////////////////////////////////////////////////

if (typeof(openerp) == "undefined") {
    openerp = new Object;
}

if (typeof(openerp.workflow) == "undefined") {
    openerp.workflow = new Object;
}

openerp.workflow.ConnectionDecorator = new Class;
openerp.workflow.ConnectionDecorator.prototype = $merge(openerp.workflow.ConnectionDecorator.prototype, draw2d.ArrowConnectionDecorator.prototype);

openerp.workflow.ConnectionDecorator.implement({
	
	initialize : function() {
		draw2d.ArrowConnectionDecorator(this);
		this.setColor(new draw2d.Color(100, 100, 100));
		this.setBackgroundColor(new draw2d.Color(100, 100, 100))
	},
	
	paint : function(/*:draw2d.Graphics*/ g) {
		
		if(this.backgroundColor!=null)
  		{
     		g.setColor(this.backgroundColor);
     		g.fillPolygon([1,6,6,1],[0,3,-3,0]);
  		}

		 // draw the border
		g.setColor(this.color);
		g.setStroke(1);
		g.drawPolygon([1,6,6,1],[0,3,-3,0]);
	}
	
	
});

// vim: ts=4 sts=4 sw=4 si et

