
.tabber { /* hide on load */
    display: none;
}

.tabberlive .tabbertabhide {
    display:none;
}

.tabberlive .tabbertab {
    background:#FBFBFB none repeat scroll 0%;
    border: 1px solid #bbb8a9;
    border-top: none;
    padding: 4px;
}

ul.tabbernav {
    list-style: none;
    padding-left: 10px;
    padding-bottom: 10px;
    border-bottom: 1px solid #bbb8a9;
    margin:0;
    height: 11px;
}

ul.tabbernav li {
    float: left;
    display: block;
    position: relative;
    height: 21px;
    padding-right: 22px;
    margin-left: -10px;
    background: url(/static/images/tabs_right_inactive.gif) 100% 0 no-repeat;
    white-space: nowrap;
}

ul.tabbernav li a {
    float: left;
    height: 21px;
    line-height: 21px;
    padding-left: 8px;
    display:block;
    background: url(/static/images/tabs_left_inactive.gif) no-repeat;
    position: relative;
}

ul.tabbernav li.tabberactive {
    z-index: 1;
    height: 22px;
    background-image: url(/static/images/tabs_right_active.gif);
}

ul.tabbernav li.tabberactive a {
    background-image: url(/static/images/tabs_left_active.gif);
    height: 22px;
    line-height: 22px;
}

ul.tabbernav li a {
    color: #222;
}

ul.tabbernav li a:hover {
    border: none;
}

/**
 * vim: sts=4 st=4 et 
 *
 */
