table.tooltip {
    position: absolute;
    display: none;
    font-family: sans-serif;
    font-size: 11px;
    color: white;
    z-index: 1000;
    max-width: 500px;
    min-width: 200px;
}

table.tooltip .tip-tl,
table.tooltip .tip-tr,
table.tooltip .tip-bl,
table.tooltip .tip-br {
    width: 9px;
    height: 9px;
    white-space: nowrap;
}

table.tooltip .tip-tl {
    background: url(/static/images/tips/tl.png) no-repeat left;
}

table.tooltip .tip-tr {
    background: url(/static/images/tips/tr.png) no-repeat right;
}

table.tooltip .tip-t {
    background: url(/static/images/tips/t.png);
}

table.tooltip .tip-bl {
    background: url(/static/images/tips/bl.png) no-repeat left;
}

table.tooltip .tip-br {
    background: url(/static/images/tips/br.png) no-repeat right;
}

table.tooltip .tip-b {
    background: url(/static/images/tips/b.png);
}

table.tooltip .tip-l {
    background: url(/static/images/tips/l.png) repeat-y left;
}

table.tooltip .tip-r {
    background: url(/static/images/tips/r.png) repeat-y right;
}

table.tooltip .tip-text {
    background: url(/static/images/tips/l.png);
}

table.tooltip .tipTitle {
    display: block;
    text-align: center;
    font-weight: bold;
    font-size: 11px;
    text-align: center;
    white-space: nowrap;
    margin-bottom: 2px;
}

table.tooltip .tipText {
    margin: 0px;
    margin-bottom: 4px;
    margin-right: 2px;
}

